#!/usr/bin/perl

##############################################################################
#  Copyright (C) 2008 Paulo Cesar Pereira de Andrade.  All Rights Reserved.
#
#  This is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This software is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  Authors:
#	Paulo Cesar Pereira de Andrade <pcpa@mandriva.com.br>
##############################################################################

use Cwd;
use Getopt::Std;

use strict;

my %options;

my @pkgconfig =
(
 "atk",
 "audiofile",
 "avahi-client",
 "avahi-glib",
 "cairo",
 "cairo-ft",
 "cairo-xlib",
 "cairo-xlib-xrender",
 "dbus-1",
 "dbus-glib-1",
 "direct",
 "directfb",
 "esound",
 "freetype2",
 "gail",
 "gamin",
 "gconf-2.0",
 "gdk-2.0",
 "gdk-pixbuf-2.0",
 "gdk-pixbuf-xlib-2.0",
 "gdk-x11-2.0",
 "glib-2.0",
 "gmodule-2.0",
 "gmodule-no-export-2.0",
 "gnome-desktop-2.0",
 "gnome-keyring-1",
 "gnome-vfs-2.0",
 "gnome-window-settings-2.0",
 "gobject-2.0",
 "gthread-2.0",
 "gtk+-2.0",
 "gtk+-x11-2.0",
 "hal",
 "lcms",
 "libart-2.0",
 "libcroco-0.6",
 "libcrypto",
 "libglade",
 "libgnome-2.0",
 "libgnomecanvas-2.0",
 "libgnome-menu",
 "libgnomeui-2.0",
 "libgsf-gnome-1",
 "libmetacity-private",
 "libpanelapplet-2.0",
 "libpng12",
 "librsvg-2.0",
 "libssl",
 "libstartup-notification-1.0",
 "libusb",
 "libwnck-1.0",
 "libxml-2.0",
 "libxslt",
 "openssl",
 "ORBit-2.0",
 "pangocairo",
 "pangoft2",
 "pango",
 "qt-mt",
 "sdl",
 );

#   Some file to check, so that build should not fail at some
# random point due to some missing requirement.
my @binaries =
(
 "aclocal",
 "ar",
 "as",
 "autoconf",
 "c++",
 "cc",
 "cpp",
 "dcopidl",		# compiz
 "dcopidl2cpp", 	# compiz
 "docbook2html",	# fontconfig
 "docbook2man", 	# fontconfig
 "docbook2pdf", 	# fontconfig
 "docbook2txt", 	# fontconfig
 "dot", 		# libxcb
 "doxygen",		# libxcb
 "ed",			# libXaw
 "ld",
 "lex",
 "git",
 "gperf",		# xcb/util
 "grep",
 "gzip",
 "iconv",		# xkbeyboard-config, compiz
 "intltoolize", 	# xkbeyboard-config, compiz
 "jw",			# fontconfig
 "kde-config",		# compiz
 "lynx",		# fontconfig
 "m4",
 "make",
 "msgfmt",		# xkbeyboard-config, compiz
 "nm",			# libXau
 "onsgmls",		# fontconfig
 "openjade",		# fontconfig
 "pkg-config",
 "ranlib",
 "sed",
 "strip",
 "tar",
 "xgettext",
 "xmllint",
 "xmlto",
 "xsltproc",
 "yacc",
 );

my @files =
(
 "/etc/sgml/sgml.conf",
 "/etc/sgml/catalog",
 "/etc/xml/catalog",
 "/usr/include/assert.h",
 "/usr/include/ctype.h",
 "/usr/include/dcopclient.h",
 "/usr/include/regex.h",
 "/usr/include/rpc/rpc.h",
 "/usr/include/stdint.h",
 "/usr/include/sys/ioctl.h",
 "/usr/lib/qt3/bin/qmake",
 );

my @libraries =
(
 "libaa",
 "libacl",
 "libart_lgpl",
 "libaudit",
 "libcrypt",
 "libdbus-qt",
 "libexpat",
 "libexslt",
 "libgcrypt",
 "libgettextlib",
 "libgg",
 "libgpg-error",
 "libgraph",
 "libidn",
 "libintl",
 "libjpeg",
 "libkdecorations",
 "libkdecore",
 "libldap",
 "libmng",
 "libncurses",
 "libncursesw",
 "libogrove",
 "libpam",
 "libpopt",
 "libpthread",
 "libsasl2",
 "libslang",
 "libtermcap",
 "libz",
 );


##############################################################################
# From xorg-build.pl
##############################################################################
sub git_config {
    my ($option) = @_;

    $option = `git-config --global --get $option`;
    $option =~ s/^\s+//;
    $option =~ s/\s+$//;

    return $option;
};

my $x_depsdir = git_config("xorg-git.deps-dir");
$x_depsdir = "/usr/local/xorg/deps" unless ($x_depsdir);
$x_depsdir = Cwd::realpath($x_depsdir);
##############################################################################
# End From xorg-build.pl
##############################################################################

##############################################################################
# Check for command line options
Getopt::Std::getopts("bh?", \%options);
if ($options{'h'} or $options{'?'}) {
    print("Usage:\n",
	  File::Basename::basename($0), " [options ...]\n",
	  " -b       Build external dependency information.\n",
	  " -h, -?   Prints this message and quits.\n");
    exit(0);
}

##############################################################################
# This can use a significant amount of memory
my %database;
sub load_database {
    my ($name);

    foreach $name (split('\n', `find $x_depsdir -type f -a -name \\*.deps.ext`)) {
	$name = Cwd::realpath($name);
	open(DATA, "<$name") or die("Cannot read $name ($!)");
	$name =~ s/^$x_depsdir\///;
	$name =~ s/\.deps.ext$//;
	while (<DATA>) {
	    chomp;
	    push(@{$database{$_}}, $name);
	}
	close(DATA);
    }
};

# Building new database?
if ($options{'b'}) {
    load_database();
    print("my %dependencies = (\n");
    foreach my $filename (sort keys %database) {
	my $count = 0;
	print("    \"$filename\"	=> [");
	foreach my $module (@{$database{$filename}}) {
	    print(", ") if (++$count > 1);
	    print("\"$module\"");
	}
	print("],\n");
    }
    print(");\n");
}
# Just check/warn if not all components required are available
else {
    foreach my $package (@pkgconfig) {
	`pkg-config --exists $package`;
	print(STDERR "pkg-config: $package is missing.\n") if ($?);
    }

    foreach my $binary (@binaries) {
	`which $binary > /dev/null 2>&1`;
	print(STDERR "binary $binary is missing.\n") if ($?);
    }

    foreach my $file (@files) {
	print(STDERR "file $file is missing.\n") unless (-f $file);
    }

    foreach my $library (@libraries) {
	foreach my $path ("/usr/lib", "/lib", "/usr/local/lib") {
	    `ls $path/$library*.so* > /dev/null 2>&1`;
	    last unless ($?);
	}
	print(STDERR "library $library is missing.\n") if ($?);
    }
}
